/*
 * kbiff.cpp
 * Copyright (C) 1999-2011 Kurt Granroth <granroth@kde.org>
 *
 * This file contains the implementation of the main KBiff
 * widget
 */
#include "kbiff.h"
#include "kbiff.moc"
#include <klocalizedstring.h>
#include <KMessageBox>
#include <KPassivePopup>
#include <KRun>
#include <KActionCollection>
#include <KAction>
#include <KMenu>
#include <KToolInvocation>
#include <QMouseEvent>
#include <knotification.h>
#include <QMovie>

#include "status.h"
#include "led.h"

#define FREE_PTRLIST(list)  while (!list.isEmpty()) delete list.takeFirst()

KBiff::KBiff(QWidget *parent_)
    : QLabel(parent_),
      statusTimer(0),
      status(NULL),
      statusChanged(true),
      mled( new Led("mled") )
{
    setMargin(0);
    setAlignment(Qt::AlignLeft | Qt::AlignTop);
    // enable the session management stuff
    connect(kapp, SIGNAL(saveYourself()), this, SLOT(saveYourself()));

    reset();
}

KBiff::~KBiff()
{
    FREE_PTRLIST(monitorList);
    delete status;
}

void KBiff::processSetup(const KBiffSetup* setup_, bool run_)
{
    // General settings
    isSecure    = setup_->getSecure();
    profile     = setup_->getProfile();
    mailClient  = setup_->getMailClient();
    sessions    = setup_->getSessionManagement();
    skipcheck   = setup_->getCheckStartup();
    noMailIcon  = setup_->getNoMailIcon();
    newMailIcon = setup_->getNewMailIcon();
    oldMailIcon = setup_->getOldMailIcon();
    noConnIcon  = setup_->getNoConnIcon();
    stoppedIcon = setup_->getStoppedIcon();

    // New mail
    systemBeep     = setup_->getSystemBeep();
    runCommand     = setup_->getRunCommand();
    runCommandPath = setup_->getRunCommandPath();
    runResetCommand     = setup_->getRunResetCommand();
    runResetCommandPath = setup_->getRunResetCommandPath();
    playSound      = setup_->getPlaySound();
    playSoundPath  = setup_->getPlaySoundPath();
    notify         = setup_->getNotify();
    dostatus       = setup_->getStatus();

    if( !dostatus )
        setToolTip(profile);
    else
        setToolTip(QString::null);

    // set all the new mailboxes
    setMailboxList(setup_->getMailboxList(), setup_->getPoll());

    // change the dock state if necessary
    if (docked != setup_->getDock())
        dock();
    else if( !docked )
        this->show();

    if (run_ && !skipcheck)
        start();
    skipcheck = false;

    // handle session management disabling
    if (sessions == false)
    {
        disconnect(this, SLOT(saveYourself()));
        kapp->disableSessionManagement();
    }

    if( status )
        setupMenuItems(status->contextMenu()->actions());
    delete setup_;
}

void KBiff::setMailboxList(const QList<KBiffMailbox*>& mailbox_list, unsigned int poll)
{
    QList<KBiffMailbox*> tmp_list = mailbox_list;

    myMUTEX = true;
    if (isRunning())
        stop();
    FREE_PTRLIST(monitorList);
    
    KBiffMailbox *mbox;
    foreach(mbox, tmp_list)
    {
        KBiffURL *url = &(mbox->url);
        KBiffMonitor *monitor = new KBiffMonitor();
        monitor->setMailbox(*url);
        monitor->setPollInterval(poll);
        monitor->setMailboxKey(mbox->key);
        connect(monitor, SIGNAL(signal_newMail(const int, const QString&)),
                this, SLOT(haveNewMail(const int, const QString&)));
        connect(monitor, SIGNAL(signal_noMail()), this, SLOT(displayPixmap()));
        connect(monitor, SIGNAL(signal_noMail()),
                this, SLOT(haveNoNewMail()));
        connect(monitor, SIGNAL(signal_oldMail()), this, SLOT(displayPixmap()));
        connect(monitor, SIGNAL(signal_oldMail()),
                this, SLOT(haveNoNewMail()));
        connect(monitor, SIGNAL(signal_noConn()), this, SLOT(displayPixmap()));
        connect(monitor, SIGNAL(signal_noConn()),
                this, SLOT(haveNoNewMail()));
        connect(monitor, SIGNAL(signal_invalidLogin(const QString&)),
                this, SLOT(invalidLogin(const QString&)));
        connect(monitor, SIGNAL(signal_fetchMail(const QString&)),
                this, SLOT(slotLaunchFetchClient(const QString&)));
        monitorList.append(monitor);
    }
    myMUTEX = false;
}

bool KBiff::isDocked() const
{
    return docked;
}

void KBiff::readSessionConfig()
{
    KConfig *config = kapp->sessionConfig();
    KConfigGroup config_group = config->group("KBiff");

    profile = config_group.readEntry("Profile", "Inbox");
    docked = config_group.readEntry("IsDocked", false);
    bool run = config_group.readEntry("IsRunning", true);

    KBiffSetup *setup_dlg = new KBiffSetup(profile);
    docked = !setup_dlg->getDock(); //processSetup will switch the 'docked' flag
                                    //back to the right value
    processSetup(setup_dlg, run);
}

///////////////////////////////////////////////////////////////////////////
// Protected Virtuals
///////////////////////////////////////////////////////////////////////////
void KBiff::mousePressEvent(QMouseEvent *e)
{
    if(e->button() == Qt::RightButton)
    {
        // popup the context menu
        popupMenu();
    }
    else
    {
        // execute the command
        slotLaunchMailClient();

        readPop3MailNow();
    }
}

///////////////////////////////////////////////////////////////////////////
// Protected Slots
///////////////////////////////////////////////////////////////////////////
void KBiff::saveYourself()
{
    if (sessions)
    {
        KConfig *config = kapp->sessionConfig();
        KConfigGroup config_group = config->group("KBiff");

        config_group.writeEntry("Profile", profile);
        config_group.writeEntry("IsDocked", docked);
        config_group.writeEntry("IsRunning", isRunning());

        config->sync();

    }
}

void KBiff::invokeHelp()
{
    KToolInvocation::invokeHelp();
}

void KBiff::displayPixmap()
{
    if (myMUTEX)
        return;

    // we will try to deduce the pixmap (or gif) name now.  it will
    // vary depending on the dock and mail state
    QString pixmap_name;
    KStatusNotifierItem::ItemStatus new_status = KStatusNotifierItem::Passive;
    bool has_new = false, has_old = false, has_no = true, has_noconn = false;
    KBiffMonitor* monitor;
    foreach(monitor, monitorList) 
    {
        switch (monitor->getMailState())
        {
        case NoMail:
            has_no = true;
            break;
        case NewMail:
            has_new = true;
            break;
        case OldMail:
            has_old = true;
            break;
        case NoConn:
            has_noconn = true;
            break;
        default:
            has_no = true;
            break;
        }
    }

    if ( !isRunning() )
    {
        pixmap_name = stoppedIcon;
        new_status= KStatusNotifierItem::Passive;
        mled->Off();
    }
    else if (has_new)
    {
        pixmap_name = newMailIcon;
        // turn on led for new mail, otherwise turn off
        mled->On();
        new_status= KStatusNotifierItem::NeedsAttention;
    }
    else if (has_old)
    {
        pixmap_name = oldMailIcon;
        new_status= KStatusNotifierItem::Active;
        mled->Off();
    }
    else if (has_noconn)
    {
        pixmap_name = noConnIcon;
        new_status= KStatusNotifierItem::NeedsAttention;
        mled->Off();
    }
    else
    {
        pixmap_name = noMailIcon;
        new_status= KStatusNotifierItem::Passive;
        mled->Off();
    }

    if( status )
    {
        QString fname= KIconLoader::global()->iconPath(pixmap_name, KIconLoader::Panel);
        if( new_status == KStatusNotifierItem::NeedsAttention )
        {
            // default to an icon
            status->setAttentionIconByName(fname); //avoid icon blinking
            // but switch to an animated movie, if necessary
            QMovie* movie= new QMovie(this);
            movie->setFileName(fname);
            if( movie->isValid() && movie->frameCount() > 1)
                status->setAttentionMovieByName(fname);
        }
        status->setIconByName(fname);

        if( dostatus )
            status->setToolTip(QIcon(kapp->windowIcon().pixmap(48,48)), profile, getStatusText());
        if( status->status() != new_status )
            status->setStatus(new_status);
        setupMenuItems(status->contextMenu()->actions());
    }

    if ( !docked )
    {
        QMovie* movie= new QMovie(this);
        QString fname= KIconLoader::global()->iconPath(pixmap_name, KIconLoader::Panel);
        movie->setFileName(fname);
        if( movie->isValid() && movie->frameCount() > 1)
        {
            setMovie(movie);
            movie->start();
        }
        else
        {
            delete movie;
            setPixmap(fname);
        }
    }
    if( dostatus )
        setToolTip(getStatusText());
    adjustSize();    
}

void KBiff::haveNewMail(const int new_num, const QString& the_mailbox)
{
    displayPixmap();

    // beep if we are allowed to
    if (systemBeep)
    {
        kapp->beep();
    }

    // run a command if we have to
    if (runCommand)
    {
        // make sure the command exists
        if (!runCommandPath.isEmpty())
        {
            executeCommand(replaceCommandArgs(runCommandPath));
        }
    }

    // play a sound if we have to
    if (playSound)
        slotPlaySound(playSoundPath);

    // notify if we must
    if (notify)
    {
        if( status )
        {
            KNotification *notification= new KNotification( "newmail", this, 
                                                            KNotification::Persistent );
            QString msg;
            msg = i18n("New Messages: %1").arg(new_num) + "<br>";
            msg += i18n("Mailbox: %1").arg(the_mailbox);

            notification->setText(msg);
            notification->setTitle(i18n("You have new mail!"));            
            KIconLoader kiconLoader("kbiff");
            notification->setPixmap(kiconLoader.loadIcon(newMailIcon, KIconLoader::Dialog));
            QStringList actions;
            actions << i18n("OK") << i18n("Mailer");
            notification->setActions( actions );
            connect(notification, SIGNAL(activated(unsigned int)), 
                    this, SLOT(notificationActivated(unsigned int)));
            notification->sendEvent();
        }
        else
        {
            QString msg;
            msg = i18n("New Messages: %1").arg(new_num) + "<br>";
            msg += i18n("Mailbox: %1").arg(the_mailbox);

            KPassivePopup::message( i18n("You have new mail!"), msg, this );
        }
    }
}

void KBiff::haveNoNewMail()
{
    displayPixmap();

    // run a command if we have to
    if (runResetCommand)
    {
        // make sure the command exists
        if (!runResetCommandPath.isEmpty())
        {
            executeCommand(runResetCommandPath);
        }
    }
}

QString KBiff::getURLWithNewMail()
{
    KBiffMonitor* monitor;
    foreach(monitor, monitorList)
    {
        if(monitor->getMailState() == NewMail)
            return monitor->getMailbox();
    }

    return monitorList.first()->getMailbox();
}

QString KBiff::getMailBoxWithNewMail()
{
    QString url(getURLWithNewMail());
    
    int slashPos = url.indexOf('/');
    if(slashPos == -1)
        return url.mid(slashPos + 1);
    else
        return url.mid(url.indexOf(':') + 1);
}

QString KBiff::replaceCommandArgs(QString cmdStr)
{
    bool expand = false;
    for(int i = 0; i < cmdStr.length(); i++)
    {
        if(expand)
        {
            expand = false;
            if(cmdStr[i] == 'm')
                cmdStr.replace(i - 1, 2, getMailBoxWithNewMail());
            else if(cmdStr[i] == 'u')
                cmdStr.replace(i - 1, 2, getURLWithNewMail());
            else if(cmdStr[i] == '%')
                cmdStr.replace(i - 1, 2, "%");

            continue;
        }

        if(cmdStr[i] == '%')
            expand = true;
    }

    return cmdStr;
}

void KBiff::dock()
{
    // destroy the old window
    if (this->isVisible())
    {
        this->hide();

        // we don't want a "real" top widget if we are _going_ to
        // be docked.
        if (docked)
            kapp->setTopWidget(this);
        else
            kapp->setTopWidget(new QWidget);
    }

    if (docked == false)
    {
        docked = true;
        // enable docking
        delete status;
        status = new KStatusNotifierItem(profile);
        status->setCategory(KStatusNotifierItem::Communications);
        status->setTitle(profile);
        status->setIconByName(KIconLoader::global()->iconPath(noMailIcon, KIconLoader::Panel));
        status->setAttentionIconByName(KIconLoader::global()->iconPath(newMailIcon, KIconLoader::Panel));
        status->setStatus(KStatusNotifierItem::NeedsAttention);
        connect(status, SIGNAL(activateRequested(bool, const QPoint&)),
                this, SLOT(activateRequested(bool, const QPoint&)));
        constructMenu(status->contextMenu());
        status->contextMenu()->removeAction(status->contextMenu()->actions().first());
        status->contextMenu()->addTitle(KIcon("kbiff", NULL), profile, 
                                        status->contextMenu()->actions().first());
        if( dostatus )
        {
            status->setToolTip(QIcon(kapp->windowIcon().pixmap(32,32)), profile, getStatusText());
        }
    }
    else
    {
        // undock it!
        delete status;
        status = NULL;
        this->show();
        docked = false;
    }
    QTimer::singleShot(1000, this, SLOT(displayPixmap()));
}

void KBiff::setup()
{
    KBiffSetup* setup_dlg = new KBiffSetup(profile);

    if (setup_dlg->exec())
        processSetup(setup_dlg, true);
    else
        delete setup_dlg;
}

void KBiff::checkMailNow()
{
    KBiffMonitor *monitor;
    foreach(monitor, monitorList)
    {
        monitor->checkMailNow();
    }
}

void KBiff::readMailNow()
{
    KBiffMonitor *monitor;
    foreach(monitor, monitorList)
    {
        monitor->setMailboxIsRead();
    }
}

void KBiff::readPop3MailNow()
{
    KBiffMonitor *monitor;
    foreach(monitor, monitorList)
    {
        if (monitor->getProtocol() == "pop3")
            monitor->setMailboxIsRead();
    }
}

void KBiff::stop()
{
    KBiffMonitor* monitor;
    foreach(monitor, monitorList)
    {
        monitor->stop();
    }
    displayPixmap();
}

void KBiff::start()
{
    myMUTEX = true;
    KBiffMonitor *monitor;
    for (int i = 0; i < monitorList.count(); i++)
    {
        monitor = monitorList.at(i);
        monitor->start();
    }
    myMUTEX = false;
    displayPixmap();
}

///////////////////////////////////////////////////////////////////////////
// Protected Functions
///////////////////////////////////////////////////////////////////////////
void KBiff::popupMenu()
{
    KMenu *popup = constructMenu();
    popup->addTitle(KIcon("kbiff", NULL), profile, popup->actions().first());
    popup->popup(QCursor::pos());
}

void KBiff::reset()
{
    // reset all the member variables
    systemBeep     = true;
    runCommand     = false;
    runCommandPath = "";
    playSound      = false;
    playSoundPath  = "";
    notify         = true;
    dostatus       = true;

    noMailIcon  = "nomail";
    newMailIcon = "newmail";
    oldMailIcon = "oldmail";
    noConnIcon  = "noconn";
    stoppedIcon = "stopped";

    docked    = false;
    isSecure  = false;

    mailClient  = "xmutt -f +%m";

    myMUTEX = false;
}

bool KBiff::isRunning()
{
    bool is_running = false;
    KBiffMonitor* monitor;
    foreach(monitor, monitorList)
    {
        if(monitor->isRunning())
        {
            is_running = true;
            break;
        }
    }
    return is_running;
}

void KBiff::executeCommand(const QString& command)
{
    KRun::runCommand(command, (QWidget*)NULL);
}

void KBiff::slotLaunchFetchClient(const QString& fetchClient)
{
    if (!fetchClient.isEmpty())
        executeCommand(fetchClient);
}

void KBiff::slotLaunchMailClient()
{
    if (!mailClient.isEmpty())
        executeCommand(replaceCommandArgs(mailClient));
}

void KBiff::slotPlaySound(const QString& play_sound)
{
    // make sure something is specified
    Phonon::MediaObject* media= Phonon::createPlayer(Phonon::NotificationCategory,
                                                     play_sound);
    media->play();
    connect( media, SIGNAL( finished() ), media, SLOT( deleteLater() ) );
}

void KBiff::invalidLogin(const QString& mailbox)
{
  QString title(i18n("Invalid Login to %1").arg(mailbox));
  KMessageBox::sorry(0,
    i18n("I was not able to login to the remote server.\n"
         "This means that either the server is down or you have "
         "entered an incorrect username or password.\n"
         "Please make sure that you have entered the correct settings."),
    title);
}

void KBiff::activateRequested(bool /*flag*/, const QPoint& /*x*/)
{
    emit slotLaunchMailClient();
}

QString KBiff::getStatusText()
{
    QString status_text;
    KBiffMonitor* mbox;
    int i=0;
    foreach(mbox, monitorList)
    {
        if( i++ )
            status_text += "<br>";
        if (mbox->newMessages() == -1)
        {
            status_text += mbox->getMailboxKey()
                + i18n(" Disabled");
        }
        else
        {
            QString key = mbox->getMailboxKey();
            key += "  ";
            status_text += key
                + QString().setNum(mbox->newMessages()) + " "
                + QString().setNum(mbox->curMessages());
        }
    }
    return status_text;
}


void KBiff::notificationActivated(unsigned int n)
{
    if(n==2)
        slotLaunchMailClient();
}

KMenu* KBiff::constructMenu(KMenu* m)
{
    bool new_menu= m==NULL;
    if( new_menu )
        m= new KMenu;
    // if secure, disable everything but exit
    if (isSecure == false)
    {
        if( new_menu )
        {            
            m->addAction(i18n("&Dock"), this, SLOT(dock()));
        }
        else
        {
            m->addAction(i18n("&UnDock"), this, SLOT(dock()));
        }

        m->addAction(i18n("&Setup..."), this, SLOT(setup()));
        m->addSeparator();
        m->addAction(KStandardAction::help(this, SLOT(invokeHelp()), m));
        m->addSeparator();
        m->addAction(i18n("&Check Mail Now"), this, SLOT(checkMailNow()))->setData(1);
        m->addAction(i18n("&Read Mail Now"), this, SLOT(readMailNow()))->setData(2);
        m->addAction(i18n("&Stop"), this, SLOT(stop()))->setData(3);
        setupMenuItems(m->actions());
    }

    if( new_menu )
    {
        m->addSeparator();
        m->addAction(KStandardAction::quit(kapp, SLOT(quit()), m));
    }
    
    return m;
}

void KBiff::setupMenuItems(QList<QAction*> actions)
{
    enum item_id { check_id=1, read_id=2, stop_id=3 };
    foreach(QAction* act, actions)
    {
        switch(act->data().toInt())
        {
        case check_id:
        case read_id:
            act->setEnabled(isRunning());
            break;
        case stop_id:
            act->disconnect(SIGNAL(triggered()));
            if( isRunning() )
            {
                connect(act, SIGNAL(triggered()), this, SLOT(stop()));
                act->setText("&Stop");
            }
            else
            {
                connect(act, SIGNAL(triggered()), this, SLOT(start()));
                act->setText("&Start");
            }
            break;
        default:
            break;
        }
    }
}
